@@ -24,8 +24,7 @@
 extern int desktop_use_shaped_icons;	/* Whether to use shaped icons or not (for slow X servers) */
 extern int desktop_auto_placement;	/* Whether to auto-place icons or not (user placement) */
 extern int desktop_snap_icons;		/* Whether to snap icons to the grid or not */
-
-extern int tree_panel_visible;	        
+extern char *desktop_directory;
 
 /* Initializes the desktop -- init DnD, load the default desktop icons, etc. */
 void desktop_init (void);
@@ -47,11 +46,18 @@ typedef struct {
 	int tmp_selected : 1;		/* Temp storage for original selection while rubberbanding */
 } DesktopIconInfo;
 
-void desktop_icon_info_destroy   (DesktopIconInfo *dii);
 void desktop_icon_info_open      (DesktopIconInfo *dii);
 void desktop_icon_info_delete    (DesktopIconInfo *dii);
 
 file_entry *file_entry_from_file (char *filename);
 void        file_entry_free      (file_entry *fe);
 
+gboolean    is_mountable (char *filename, file_entry *fe, int *is_mounted, char **mount_point);
+gboolean    do_mount_umount (char *filename, gboolean is_mount);
+
+void desktop_arrange_icons (void);
+void desktop_rescan_devices (void);
+void desktop_reload_icons (int user_pos, int xpos, int ypos);
+
+extern int desktop_wm_is_gnome_compliant;
 #endif
